#include <math.h>
#include "../pablio0.1/include/pablio.h"

#define FREQUENCY      440.0

#define M_PI 3.141592654

const int bufsz = 1000;

double level = .1;

float buffer[bufsz];
float force[bufsz];

double maxSignal=0;

// hit bell every 3 seconds
double tollT = 3;

int srate = 44100;

// define ff[] dd[] aa[] with nmodes=50 modes
#include "bellmodel.h"


/** State of filters. */
double yt_1[nmodes], yt_2[nmodes]; 

/** The transfer function of a reson filter is H(z) = 1/(1-twoRCosTheta/z + R2/z*z). */
double R2[nmodes];

double twoRCosTheta[nmodes];

/** Reson filter gain. */
double ampR[nmodes]; 

/** Cached values. */
double c_i[nmodes];

/** Scratch buffer. */
double scratchBuf[nmodes];


/** Compute the reson coefficients from the modal model parameters.
*/
void computeResonCoeff() {
    for(int i=0;i<nmodes;i++) {
        double tmp_r = (double)(exp(-dd[i]/srate));
        R2[i] = tmp_r*tmp_r;
        twoRCosTheta[i] = (double)(2*cos(2*M_PI*ff[i]/srate)*tmp_r);
        c_i[i] = (double)(sin(2*M_PI*ff[i]/srate)*tmp_r);
    }        
}

/** Compute gains.
*/
void computeLocation() {
    for(int i=0;i<nmodes;i++) {
        ampR[i] = c_i[i] * aa[i];
    }   
}


/** Compute the filter coefficients used for real-time rendering
from the modal model parameters. 
*/
void computeFilter() {
    computeResonCoeff();
    computeLocation();
}

/** Return audio force, which will usually be "applied" to the eardrum.
*/
void getForce() {
    static int n=0;
    double dt = bufsz/(double)srate; 
    int nT = (int)(tollT/dt); // hit every nT'th time
    if(n == nT) {
        n = 0;
        force[0] = level;
    } else {
        n++;
        force[0] = 0;
    }
}

/** Apply external force[] and compute response.

*/
void computeSoundBuffer() {
    int k,i;
    for(k=0;k<bufsz;k++) {
        buffer[k] = 0;
    }
    for(i=0;i<nmodes;i++) {
        double tmp_twoRCosTheta = twoRCosTheta[i];
        double tmp_R2 = R2[i];
        double tmp_a = ampR[i];
        double tmp_yt_1 = yt_1[i];
        double tmp_yt_2 = yt_2[i];
        for(k=0;k<bufsz;k++) {
            double ynew = tmp_twoRCosTheta * tmp_yt_1 -
                tmp_R2 * tmp_yt_2 + tmp_a * force[k];
            tmp_yt_2 = tmp_yt_1;
            tmp_yt_1 = ynew;
            buffer[k] += ynew;
        }
        yt_1[i] = tmp_yt_1;
        yt_2[i] = tmp_yt_2;
    }
}

int main(int argc, char* argv[])
{
    PABLIO_Stream  *outStream;
    maxSignal = 0;
    int i;
    for(i=0;i<nmodes;i++) {
        yt_1[i] = yt_2[i] = 0;
    }
    for(i=0;i<bufsz;i++) {
        force[i] = buffer[i] = 0;
    }
    computeFilter();
    OpenAudioStream(&outStream, (double) srate, paFloat32, PABLIO_WRITE|PABLIO_MONO);
    
    while(1) {
        getForce();
        computeSoundBuffer();
        WriteAudioStream(outStream, buffer, bufsz); 
    }
    return 0;
}

