import javax.sound.sampled.*;
import java.io.*;

/**
   A buffer loaded from an audio file.
   Copyright The University of British Columbia, Canada
   @author Kees van den Doel (kvdoel@cs.ubc.ca)
*/
public class AudioFileBuffer {
    /** Sampling rate  in Hertz. */
    public float srate;

    /** Length of looping buffer. */
    public int bufsz;

    /** Looping buffer. */
    public float[] buf;

    /** Construct buffer from named file.
        @param fn Audio file name.
    */
    public AudioFileBuffer(String fn) {
        loadAudio(fn);
    }

    /** Load audio file.
        @param fn Audio file name.
    */
    public void loadAudio(String fn) {
        int totalFramesRead = 0;
        File fileIn = new File(fn);
        int numBytes=0;
        byte[] audioBytes = null;
        try {
            AudioInputStream audioInputStream = AudioSystem.getAudioInputStream(fileIn);
            int bytesPerFrame = audioInputStream.getFormat().getFrameSize();
            this.srate = (float)audioInputStream.getFormat().getSampleRate();
            long nFrames = audioInputStream.getFrameLength();
            numBytes = (int)nFrames * bytesPerFrame; 
            audioBytes = new byte[numBytes];
            try {
                int numBytesRead = 0;
                int numFramesRead = 0;
                while ((numBytesRead = audioInputStream.read(audioBytes)) != -1);
                audioInputStream.close();
            } catch (Exception ex) {
                System.out.println("Error loading audio file " + fn);
            }
        } catch (Exception e) {
            System.out.println("Error reading audio file " + fn);
        }

        bufsz = numBytes/2;
        buf = new float[bufsz];
        FormatUtils.byteToFloat(buf,audioBytes,bufsz);
    }

}


