/**
   A force model  for hitting.  Duration of the  contact.  settable.  If
   contact time  is short  wil get a  sharp, metalling hit.   If contact
   duration  is  longer  get  sound  of a  rubber  hammer  for  example.
   Ballpark figure for contact duration is about 10 ms, with 50ms a very
   soft blow.
   BUG: if bufferSize is too small to contains a complete force profile,
   only as much  of the impact shape as fits is returned.
   Copyright The University of British Columbia, Canada
   @author Kees van den Doel (kvdoel@cs.ubc.ca)
*/
public class BangForce implements AudioForce {
    /** Sampling rate in Hertz. */
    private float srate;
    
    /** Strength of impact. */    
    private float magnitude=1;

    /** Number of samples of the impact. */
    private int durationInSamples=1;

    /** True if {@link #getForce} should return a hit on next call. */
    private boolean isHit = false;

    /** Constructor
        @param srate sampling rate in Hertz.
    */
    public BangForce(float srate) {
        this.srate = srate;
    }

    /** Set impact strength.
        @param magnitude strength of impact.
    */
    public void setMagnitude(float magnitude) {
        this.magnitude = magnitude;
    }
    
    /** Set  contact  time  in  seconds. Smaller  values  give  sharper
        sounds. 50ms is very soft.
        @param dur duration of contact in seconds.
    */
    public void setDuration(float dur) {
        durationInSamples = (int)(dur * srate);
        if(durationInSamples < 1) {
            durationInSamples = 1;
        }
    }

    /** Calling this method will cause next call to {@link #getForce} to
        return an impact.
    */
    public void hit() {
        isHit = true;
    }
    
    /** Create force as a single period of (1-cos).
        @param output user provided buffer for force returned.
        @param nsamples number of samples written to output buffer.
    */
    public void getForce(float [] output, int nsamples) {
        if(isHit) { // was hit
            int n;
            // Shortest impact consists of one sample
            if(nsamples < durationInSamples) {
                n = nsamples;
            } else {
                n = durationInSamples;
            }
	    // Load with cosine. Of course this is not the most 
	    // efficient way with expensive calls to Math.cos().
            for(int i=0;i<n;i++) {
                output[i] = (float)(.5*(1.-Math.cos(2*Math.PI*(i+1)/(1+durationInSamples))) * magnitude);
            }
            for(int i=n;i<nsamples;i++) {
                output[i] = 0;            
            }
            // processed hit
            isHit = false;
        } else { // was not hit
            for(int i=0;i<nsamples;i++) {
                output[i] = 0;            
            }            
        }
    }

}


