
/** Hit a bottle at different locations.
 * Copyright The University of British Columbia, Canada
 @author Kees van den Doel (kvdoel@cs.ubc.ca)
 */
public class DemoBottleHit extends java.awt.Dialog {

    /** Creates new form DemoBottleHit */
    public DemoBottleHit(java.awt.Frame parent,boolean modal) {
        super (parent, modal);
        initComponents ();
        pack ();
    }

    private void initComponents () {
        jSlider1 = new javax.swing.JSlider ();
        jSlider2 = new javax.swing.JSlider ();
        setLayout (new java.awt.BorderLayout ());
        addWindowListener (new java.awt.event.WindowAdapter () {
                public void windowClosing (java.awt.event.WindowEvent evt) {
                    closeDialog (evt);
                    System.exit(0);
                }
            }
            );

        jSlider1.setMaximum (1000000);
        jSlider2.setMaximum (1000000);
        jSlider1.addMouseMotionListener (new java.awt.event.MouseMotionAdapter () {
                public void mouseDragged (java.awt.event.MouseEvent evt) {
                    jSlider1MouseDragged (evt);
                }
            }
            );
        jSlider2.addMouseMotionListener (new java.awt.event.MouseMotionAdapter () {
                public void mouseDragged (java.awt.event.MouseEvent evt) {
                    jSlider2MouseDragged (evt);
                }
            }
            );


        add (jSlider1, java.awt.BorderLayout.CENTER);
        add (jSlider2, java.awt.BorderLayout.SOUTH);

    }

    private void jSlider1MouseDragged (java.awt.event.MouseEvent evt) {
        double val = jSlider1.getValue()/1000000.;
        double min = .0001,max=.05;
        val = val*(max-min)+min;
        System.out.println("hit duration=" + val);
        af.setDuration((float)val);
    }

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {
        setVisible (false);
        dispose ();
    }

    private void jSlider2MouseDragged (java.awt.event.MouseEvent evt) {
        double val = jSlider2.getValue()/1000000.;
        if(val == 1.) {
            val -= .00001;
        }
        if(val < 1./(sob.modalModel.np)) {
            val = .00001 + 1./(sob.modalModel.np);
        }
        System.out.println("hit position=" + val);
        double x = sob.modalModel.np * val - 1.;

        int p1 = (int)x; // pick position, normalized to the left of center
        int p2 = p1 + 1;
        int p3 = 0; // zero weight, does not matter
        x -= p1; // fractional part of location
        double b1 = x;
        double b2 = 1.-x;
        double b3 = 0;

        sob.setLocation(p1,p2,p3,(float)b1,(float)b2,(float)b3);

    }

    static ModalSonicObject sob;
    static BangForce af;
    static int np; // number of points on object

    public static void main (String args[]) {
        float srate = 44100.f;
        int bufferSize = 128;
        int bufferSizeJavaSound = 8*1024;

        try {
            sob = new ModalSonicObject(new ModalModel("calona0.sy"),
                                       srate,bufferSize,bufferSizeJavaSound);
        } catch (java.io.FileNotFoundException ee) {
            System.out.println("Modes file not found\n");
        }
        af = new BangForce(srate);
        af.setDuration(.0001f);
        sob.setAudioForce(af);
        sob.start();
        new Thread() {
                int t_control = 200; //In ms
                public void run() {
                    try {
                        while(true) {
                            sleep(t_control);
                            af.hit();
                        }
                    } catch(InterruptedException ee) {
                    }
                }
            }.start();
        new DemoBottleHit (new java.awt.Frame (), true).show ();
    }
    
    private javax.swing.JSlider jSlider1;
    private javax.swing.JSlider jSlider2;
    
}
