import java.io.*;

/** Acoustic model of object, capable of playing sound in resonse to a
    stimulus force obtained. Implements AudioForce interface, allowing
    one SonicOcject to act as a source to each other.
    Copyright The University of British Columbia, Canada
    @author Kees van den Doel (kvdoel@cs.ubc.ca)
*/

public abstract class SonicObject extends Thread implements AudioForce {
        
    /** Sampling rate in Hertz. */
    protected float srate;

    /** Buffer size used for real-time rendering. */
    protected int bufferSize;

    /** Buffer size used by JavaSound. */
    private int bufferSizeJavaSound;

    /** Use automatic gain control. */
    private boolean agc = true;

    /** Maximum value of output thus far. Used for AGC. */
    protected float maxSignal;

    /** Turn Automatic Gain Control (AGC) on or off for real-time rendering.
        @param val false to turn AGC off, default is <code>true</code>.
     */
    public void setAGC(boolean val) {
        agc = val;
    }

    /** Start synthesis thread of object, obtaining force through AudioForce object,
        and "applying" it to audio out.
     */
    public void run() {
        float [] y = new float[bufferSize];
        RTPlay pb = new RTPlay(bufferSizeJavaSound,srate,16,1,true);
        while(true) {
            this.getForce(y,bufferSize);
            if(agc) {
                for(int i=0;i<bufferSize;i++) {
                    float tmpAbs = Math.abs(y[i]);                
                    if(tmpAbs > maxSignal) {
                        maxSignal = tmpAbs;           
                    }
                }
                for(int i=0;i<bufferSize;i++) {
                    y[i] /= maxSignal;
                }
            }
            pb.write(y);
        }    
    }
    
    /** Create and initialize SonicObject.
        @param srate sampling rate in Hertz.
        @param bufferSize Buffer size used for real-time rendering.
        @param bufferSizeJavaSound Buffer size used by JavaSound.
     */
    public SonicObject(float srate,int bufferSize,int bufferSizeJavaSound) {
        this.bufferSize = bufferSize;
        this.bufferSizeJavaSound = bufferSizeJavaSound;
        this.srate = srate;
    }

}
